//                              -*- Mode: C++ -*- 
// ReferenceExistence.cc --- Reference Existence sub-hierarchy
// Author          : Nicolas Anquetil
// Created On      : Mon Dec  2 13:26:48 1996
// 

# include "ReferenceExistence.hh"

// **************************************************
//					ReferenceExistence
// **************************************************

ReferenceExistence::ReferenceExistence(char *s)
//--------------------------------------------------
{
  Name *n;

# ifdef DEBUG
  printf("ReferenceExistence::ReferenceExistence(%s)\n",s);
# endif
  n = the_DB->getName(s);
  _referenceName = n;
  n->addNamedReference( this);
  the_DB->incCount(classType());
}  /* ReferenceExistence::ReferenceExistence */


# ifdef DEBUG
void ReferenceExistence::hello()
//--------------------------------------------------
{
  printf("ReferenceExistence (%s)\nis a ", referenceName()->string());
  SoftwareObject::hello();
}  /* ReferenceExistence::hello */
# endif


// **************************************************
//					CommentTermExistence
// **************************************************

XCommentTermExistence::XCommentTermExistence( char *name,
//--------------------------------------------------
    SourceUnit *containingSource,
	bool isConditionalCompData)      // ??
  : ReferenceExistence(name)
{
# ifdef DEBUG
  printf("CommentTermExistence::CommentTermExistence(%s)\n",name);
# endif
  _foundInSource = containingSource;
  containingSource->addReference( this, true);  // true=is a comment
  the_DB->incCount(classType());
}  /* XCommentTermExistence::XCommentTermExistence */


VoidIterator<SoftwareObject*> *XCommentTermExistence::referedBy(class_t type)
//--------------------------------------------------
// May be refered to by file or routine
{
  if (foundInSource() == NULL)
	return(new VoidIterator<SoftwareObject*>());
  else
	return( new TypeFilterIterator<SourceUnit*,SoftwareObject*>(
              new SingletonIterator<SourceUnit*>( foundInSource()),
			  type) );
}  /* XCommentTermExistence::referedBy */


void XCommentTermExistence::addSuccessorInCompound(
//--------------------------------------------------
// There is no addPredecessorInCompound() function,
// This one does everything
   XCommentTermExistence *successor)
{
  addInSet( successorInCompound(), successor);
  addInSet( successor->predecessorInCompound(), this);
}  /* XCommentTermExistence::addSuccessorInCompound */


# ifdef DEBUG
void XCommentTermExistence::hello()
//--------------------------------------------------
{
  printf("CommentTermExistence (%s)\nis a ", referenceName()->string());
  ReferenceExistence::hello();
  printf("found in ----------\n");
  foundInSource()->hello();
  printf("---------- found in\n");
}  /* CommentTermExistence::hello */
# endif

// **************************************************
//					SpecialCodeExistence
// **************************************************

# ifdef DEBUG
void SpecialCodeExistence::hello()
//--------------------------------------------------
{
  printf("SpecialCodeExistence (%s)\nis a ", referenceName()->string());
  CommentTermExistence::hello();
}  /* SpecialCodeExistence::hello */
# endif


// **************************************************
//				FileInclusionExistence
// **************************************************

VoidIterator<SoftwareObject*> *FileInclusionExistence::referedBy(class_t type)
//--------------------------------------------------
// May be refered to by file
{
  return( new TypeFilterIterator<SourceFile*,SoftwareObject*>(
            new LIterator<SourceFile*>( potentiallyIncludedIn()),
			type) );
}  /* FileInclusionExistence::referedBy */


# ifdef DEBUG
void FileInclusionExistence::hello()
//--------------------------------------------------
{
  printf("FileInclusionExistence (%s)\nis a ", referenceName()->string());
  ReferenceExistence::hello();
  printf("included in %d files\n", potentiallyIncludedIn()->size());
}  /* FileInclusionExistence::hello */
# endif


// **************************************************
//					DataUseExistence
// **************************************************

VoidIterator<SoftwareObject*> *XDataUseExistence::referedBy(class_t
type)
//--------------------------------------------------
// May be refered to by file or routine
{
  return( new TypeFilterIterator<SourceUnit*,SoftwareObject*>(
            new LIterator<SourceUnit*>( foundInSource()),
			type) );
}  /* XDataUseExistence::referedBy */


void XDataUseExistence::addSuccessorInCompound(
//--------------------------------------------------
// There is no addPredecessorInCompound() function,
// This one does everything
   XDataUseExistence *successor)
{
  addInSet( successorInCompound(), successor);
  addInSet( successor->predecessorInCompound(), this);
}  /* XDataUseExistence::addSuccessorInCompound */


# ifdef DEBUG
void XDataUseExistence::hello()
//--------------------------------------------------
{
  printf("DataUseExistence (%s)\nis a ", referenceName()->string());
  ReferenceExistence::hello();
  printf("found in %d sources\n", foundInSource()->size());
}  /* XDataUseExistence::hello */
# endif


// **************************************************
//				 ManifestConstExistence
// **************************************************

# ifdef DEBUG
void ManifestConstExistence::hello()
//--------------------------------------------------
{
  printf("ManifestConstExistence (%s)\nis a ", referenceName()->string());
  DataUseExistence::hello();
}  /* ManifestConstExistence::hello */
# endif


// **************************************************
//					  TypeUseExistence
// **************************************************

VoidIterator<SoftwareObject*> *TypeUseExistence::referedBy(class_t type)
//--------------------------------------------------
// May be refered to by "same as DataUseExistence" + variable or type
{
  VoidIterator<SoftwareObject*> *iter;

  if ( isCdbSuperClass( type, _routinesource) ||
	   isCdbSuperClass( _routinesource, type) )
	iter = new TypeFilterIterator<RoutineSource*,SoftwareObject*>(
             new LIterator<RoutineSource*>( usedAsReturnTypeOf()),
	     type);
  else
	iter = new VoidIterator<SoftwareObject*>();

  iter =  new SortedUnionIterator<SoftwareObject*>(
            iter,
	    new TypeFilterIterator<TypedDefinition*,SoftwareObject*>(
              new LIterator<TypedDefinition*>( usedAsTypeOf()),
	      type),
	    SO_strict_cmp, SOmerge) ;

  return( new SortedUnionIterator<SoftwareObject*>(
            iter,

	    new TypeFilterIterator<SourceUnit*,SoftwareObject*>(
              new LIterator<SourceUnit*>( foundInSource()),
	      type),
	    SO_strict_cmp, SOmerge) );

}  /* TypeUseExistence::referedBy */


# ifdef DEBUG
void TypeUseExistence::hello()
//--------------------------------------------------
{
  printf("TypeUseExistence (%s)\nis a ", referenceName()->string());
  DataUseExistence::hello();
  printf("type of %d references\n", usedAsTypeOf()->size());
  printf("return type of %d routines\n", usedAsReturnTypeOf()->size());
}  /* TypeUseExistence::hello */
# endif


// **************************************************
//					  RoutineCallExistence
// **************************************************

VoidIterator<SoftwareObject*> *RoutineCallExistence::referedBy(class_t type)
//--------------------------------------------------
// May be refered to by routine
{
  return( new TypeFilterIterator<SourceUnit*,SoftwareObject*>(
            new LIterator<SourceUnit*>( potentiallyCalledBy()),
			type) );
}  /* RoutineCallExistence::referedBy */


# ifdef DEBUG
void RoutineCallExistence::hello()
//--------------------------------------------------
{
  printf("RoutineCallExistence (%s)\nis a ", referenceName()->string());
  ReferenceExistence::hello();
  printf("called by %d sources\n", potentiallyCalledBy()->size());
}  /* RoutineCallExistence::hello */
# endif
