//                               -*- Mode: c++ -*- 
// ReferenceExistence.hh --- Reference Existence sub-hierarchy
// Author          : Nicolas Anquetil
// Created On      : Mon Nov 25 12:17:12 1996
//


# ifndef REFERENCEEXISTENCE_H
# define REFERENCEEXISTENCE_H

# include "utils.hh"

# include "Definition.hh"
# include "SourceUnit.hh"


class ReferenceExistence: public SoftwareObject {
// --------------------------------------------------
protected:
  Name *_referenceName;
  // converse: Name -> (List) namedReferences

public:
  ReferenceExistence(char *s);

  Name *referenceName() const { return(_referenceName); };
  char *objectName()    const { return( referenceName()->string()); };

  class_t classType() { return(_referenceexistence); };
  bool is_a(class_t aType) {
	return( (aType == _referenceexistence) || SoftwareObject::is_a(aType)); };
# ifdef DEBUG
  void hello();
# endif
};


class XCommentTermExistence: public ReferenceExistence {
// --------------------------------------------------
protected:
  SourceUnit *_foundInSource;
  // converse: SourceUnit -> (List) refersToCommentTerm
  List<XCommentTermExistence *> _successorInCompound;
  // converse: CommentTermExistence -> (List) predecessorInCompound
  List<XCommentTermExistence *> _predecessorInCompound;
  // converse: CommentTermExistence -> (List) successorInCompound

public:
  XCommentTermExistence( char *name,
						SourceUnit *containingSource,
						bool isConditionalCompData);

  SourceUnit *foundInSource() { return(_foundInSource); };
  List<XCommentTermExistence *> *successorInCompound() {
	return(&_successorInCompound); };
  List<XCommentTermExistence *> *predecessorInCompound() {
	return(&_predecessorInCompound); };

  VoidIterator<SoftwareObject*> *referedBy(class_t type);
  void addSuccessorInCompound( XCommentTermExistence *successor);
  class_t classType() { return(_commenttermexistence); };
  bool is_a(class_t aType) {
	return( (aType == _commenttermexistence) || ReferenceExistence::is_a(aType)); };

  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_successorInCompound);
	sortTheList((List<SoftwareObject*>*)&_predecessorInCompound);
	ReferenceExistence::sortLists(); };

# ifdef DEBUG
  void hello();
# endif

};

class CommentTermExistence: public XCommentTermExistence {
// --------------------------------------------------
public:
  CommentTermExistence( char *name,
						SourceUnit *containingSource,
						bool isConditionalCompData):
  XCommentTermExistence (name, containingSource, isConditionalCompData) {};
};


class SpecialCodeExistence: public CommentTermExistence {
// --------------------------------------------------
public:
  SpecialCodeExistence( char *name,
						SourceUnit *containingSource,
						bool isConditionalCompData)
	: CommentTermExistence(name, containingSource, isConditionalCompData) 
{the_DB->incCount(classType());
	  /* nothing more */ };

  class_t classType() { return(_specialcodeexistence); };
  bool is_a(class_t aType) {
	return( (aType == _specialcodeexistence) || CommentTermExistence::is_a(aType)); };
# ifdef DEBUG
  void hello();
# endif
};


class FileInclusionExistence: public ReferenceExistence {
// --------------------------------------------------
protected:
  List<SourceFile *> _potentiallyIncludedIn;
  // converse: SourceFile -> (List) includes

public:
  FileInclusionExistence( char *name)
	: ReferenceExistence(name)
 { the_DB->incCount(classType());
   /* nothing more */ };

  List<SourceFile *> *potentiallyIncludedIn() {
	return(&_potentiallyIncludedIn); };

  VoidIterator<SoftwareObject*> *referedBy(class_t type);
  void addInclusion(SourceFile *includingSource) {
	addInSet( potentiallyIncludedIn(), includingSource); };
  class_t classType() { return(_fileinclusionexistence); };
  bool is_a(class_t aType) {
	return( (aType == _fileinclusionexistence) || ReferenceExistence::is_a(aType)); };

  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_potentiallyIncludedIn);
	ReferenceExistence::sortLists(); };

# ifdef DEBUG
  void hello();
# endif
};


class XDataUseExistence: public ReferenceExistence {
// --------------------------------------------------
protected:
  List<SourceUnit *> _foundInSource;
  // converse: SourceUnit -> (List) refersToData
  List<XDataUseExistence *> _successorInCompound;
  // converse: DataUseExistence -> (List) predecessorInCompound
  List<XDataUseExistence *> _predecessorInCompound;
  // converse: DataUseExistence -> (List) successorInCompound

public:
  XDataUseExistence( char *name)   
        : ReferenceExistence(name)
  {
   /* nothing more */ };

  List<SourceUnit *> *foundInSource() { return(&_foundInSource); };
  List<XDataUseExistence *> *successorInCompound() {
	return(&_successorInCompound); };
  List<XDataUseExistence *> *predecessorInCompound() {
	return(&_predecessorInCompound); };

  VoidIterator<SoftwareObject*> *referedBy(class_t type);
  void addReferingSource(SourceUnit *source) {
	addInSet( foundInSource(), source); };
  void addSuccessorInCompound( XDataUseExistence *successor);
  class_t classType() { return(_datauseexistence); };
  bool is_a(class_t aType) {
	return((aType == _datauseexistence ) || ReferenceExistence::is_a(aType)); };


  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_foundInSource);
	sortTheList((List<SoftwareObject*>*)&_successorInCompound);
	sortTheList((List<SoftwareObject*>*)&_predecessorInCompound);
	ReferenceExistence::sortLists(); };

# ifdef DEBUG
  void hello();
# endif
};

class DataUseExistence: public XDataUseExistence {
// --------------------------------------------------

public:
  DataUseExistence( char *name)
	: XDataUseExistence(name)
  { the_DB->incCount(classType());
   /* nothing more */ };


};


class ManifestConstExistence: public DataUseExistence {
// --------------------------------------------------

public:
  ManifestConstExistence( char *name)
	: DataUseExistence( name)
 { the_DB->incCount(classType());
   /* nothing more */ };

  class_t classType() { return(_manifestconstexistence); };
  bool is_a(class_t aType) {
	// implementation does not correspond to actual class hierarchy here,
    // but conceptually, ManifestConstExistence should not be a
	// DataUseExistence 
	return((aType == _manifestconstexistence ) || ReferenceExistence::is_a(aType)); };

# ifdef DEBUG
  void hello();
# endif
};


class TypeUseExistence: public DataUseExistence {
// --------------------------------------------------
protected:
  List<TypedDefinition *> _usedAsTypeOf;
  // converse: TypedDefinition -> type
  List<RoutineSource *>   _usedAsReturnTypeOf;
  // converse: RoutineSource -> returnType

public:
  TypeUseExistence( char *name)
	: DataUseExistence( name)
  { the_DB->incCount(classType());
   /* nothing more */ };

  List<TypedDefinition *> *usedAsTypeOf() { return(&_usedAsTypeOf); };
  List<RoutineSource *>   *usedAsReturnTypeOf() {
	return(&_usedAsReturnTypeOf); };

  VoidIterator<SoftwareObject*> *referedBy(class_t type);
  void addUseType( TypedDefinition *usedBy) {
	addInSet( usedAsTypeOf(), usedBy); };
  void addUseReturnType( RoutineSource *usedBy) {
	addInSet( usedAsReturnTypeOf(), usedBy); };
  class_t classType() { return(_typeuseexistence); };
	// implementation does not correspond to actual class hierarchy here,
    // but conceptually, TypeUseExistence should not be a DataUseExistence 
  bool is_a(class_t aType) {
	return((aType ==_typeuseexistence  ) || ReferenceExistence::is_a(aType)); };

  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_usedAsTypeOf);
	sortTheList((List<SoftwareObject*>*)&_usedAsReturnTypeOf);
	DataUseExistence::sortLists(); };

# ifdef DEBUG
  void hello();
# endif
};


class RoutineCallExistence: public ReferenceExistence {
// --------------------------------------------------
protected:
  List<SourceUnit *> _potentiallyCalledBy;
  // converse: RoutineSource -> (List) calls

public:
  RoutineCallExistence( char *called)
	: ReferenceExistence(called)
 { the_DB->incCount(classType());  
 /* nothing more */ };

  List<SourceUnit *> *potentiallyCalledBy() {
	return(&_potentiallyCalledBy); };

  VoidIterator<SoftwareObject*> *referedBy(class_t type);
  void addCallingSource(SourceUnit *caller) {
	addInSet( potentiallyCalledBy(), caller); };
  class_t classType() { return(_routinecallexistence); };
  bool is_a(class_t aType) {
	return( (aType == _routinecallexistence) || ReferenceExistence::is_a(aType)); };

  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_potentiallyCalledBy);
	ReferenceExistence::sortLists(); };

# ifdef DEBUG
  void hello();
# endif
};


# endif
