/*                               -*- Mode: C -*- 
 * cdbFileMem.c --- interface between CDB and FileMem utilities
 * Author          : Nicolas Anquetil
 * Created On      : Mon Feb 22 18:04:11 1999
 * Last Modified By: Nicolas Anquetil
 * Last Modified On: Thu Apr 29 19:41:50 1999
 */


# include "cdbFileMem.hh"

// tell the 3 following function wether we have a filemem opened in write mode
static bool initialized = false;
static int  openMode    = -1;


void *cdbAlloc(size_t sz)
//--------------------------------------------------
// redefinition of memory allocation to work with FileMem
// if the filemem memory block is open in read/write, takes the memory
// form it, otherwise use normal malloc
{
  void *mem;

  if (openMode == -1)
	fileMemState( (int *)&initialized, &openMode);

  if (initialized && (openMode != UFM_READ)) {
	mem= (void *)allocFileMem(sz);
# ifdef DEBUG
	printf("---IN NEW(%d), got address %x from FileMem\n", sz, mem);
# endif
  }
  else {
	mem= (void *)malloc(sz);
# ifdef DEBUG
	printf("---IN NEW(%d), got address %x from heap\n", sz, mem);
# endif
  }

  return( mem);
}  /* cdbAlloc() */


void *cdbReAlloc(void *addr, size_t sz)
//--------------------------------------------------
// redefinition of memory allocation to work with FileMem
// if the filemem memory block is open in read/write, takes the memory
// form it, otherwise use normal malloc
{
  void *mem;

  if (openMode == -1)
	fileMemState( (int *)&initialized, &openMode);

  if (initialized && (openMode != UFM_READ)) {
	mem= (void *)reallocFileMem(addr,sz);
# ifdef DEBUG
	printf("---IN NEW(%d), got address %x from FileMem\n", sz, mem);
# endif
  }
  else {
	mem= (void *)realloc(addr,sz);
# ifdef DEBUG
	printf("---IN NEW(%d), got address %x from heap\n", sz, mem);
# endif
  }

  return( mem);
}  /* cdbAlloc() */


void cdbFree(void *o)
//--------------------------------------------------
// redefinition of memory allocation to work with FileMem
{
  if (openMode == -1)
	fileMemState( (int *)&initialized, &openMode);

  if (initialized && (openMode != UFM_READ)) {
# ifdef DEBUG
	printf("---IN DELETE(%x), return address to FileMem\n", o);
	debugPrintMem((char *)o);
# endif
	freeFileMem(o);
  }

  else {
# ifdef DEBUG
	printf("---IN DELETE(%x), return address to heap\n", o);
# endif
  free(o);
  }
}  /* cdbFree() */
