//                              -*- Mode: C++ -*- 
// cdblist.hh --- A simple linked list
// Author          : Nicolas Anquetil
// Created On      : Mon Feb 22 10:45:38 1999
// 


# ifndef CONTAINER_H
# define CONTAINER_H

extern "C" {
# include <stdlib.h>
}

# define VECT_INCR 1.25  // => increment by 25%


// **************************************************
template <class TItem>
class Cell {
// **************************************************
protected:
  TItem _val;
  Cell<TItem> *_next;

public:
  Cell(TItem v): _val(v) { _next = NULL; };

# ifdef CDBFILEMEM_H
  inline void *operator new(size_t sz) { return(cdbAlloc(sz)); };
  inline void operator delete(void *o) { cdbFree(o); };
# endif

  TItem val()  { return(_val); };
  Cell<TItem> *next() { return(_next); };

  void add(Cell<TItem> *n) { _next = n; };
}; // Cell


// **************************************************
template <class TItem>
class List {
// **************************************************
protected:
  Cell<TItem> *_begin;
  Cell<TItem> *_end;
  int _size;
public:
  List() { _begin = _end = NULL; _size = 0; };
  ~List() { clear(); };

# ifdef CDBFILEMEM_H
  inline void *operator new(size_t sz) { return(cdbAlloc(sz)); };
  inline void operator delete(void *o) { cdbFree(o); };
# endif

  Cell<TItem> *begin() { return(_begin); };
  Cell<TItem> *end() { return(_end); };
  int size() { return(_size); };

  void add(TItem v) {
	if (_end == NULL)
	  _begin = _end = new Cell<TItem>(v);
	else {
  _end->add( new Cell<TItem>(v));
	  _end = _end->next();
	}
	_size++;
  };  // add()
  void clear() {
	Cell<TItem> *tmp;
	while (_begin != _end) {
	  tmp = _begin->next();
	  delete(_begin);
	  _begin = tmp;
	}
	_begin = _end = NULL; _size = 0; };  // clear()
	
};   /* class List */


// **************************************************
template <class TItem>
class Vector {
// **************************************************
protected:
  TItem *_begin;
  float  _increment;  // increment step
  int    _size;       // nb of element in vector
  int    _space;      // nb of "cells" in vector

public:
  Vector() { _begin=NULL; _size=_space=0; _increment=VECT_INCR; };
  Vector(int inc) { _begin=NULL; _size=_space=0; _increment=inc; };
  ~Vector() { clear(); delete(_begin); };

# ifdef CDBFILEMEM_H
  inline void *operator new(size_t sz) { return(cdbAlloc(sz)); };
  inline void operator delete(void *o) { cdbFree(o); };
# endif

  TItem *begin() { return(_begin); };
  TItem *_end() { return(_begin+_size-1); };
  int size() { return(_size); };

  inline TItem operator [](int i) { return(*(_begin+i)); };

  TItem at(int i) { return(*(_begin+i)); };
  void at(int i, TItem v) { _begin[i] = v; };

  void add(TItem v) {
	if (_size >= _space) {
	  if (_size == 0)
		_space = 100;
	  else
		_space = (int)((float)_space * _increment);

# ifdef CDBFILEMEM_H
	  _begin = (TItem*)cdbReAlloc(_begin, _space*sizeof(TItem));
# else
	  _begin = (TItem*)realloc(_begin, _space*sizeof(TItem));
# endif
	}
	_begin[_size] = v;
	_size++;
  };  // add()
  void clear() { _begin=NULL; _size=0; };
};

# endif
