//                              -*- Mode: C++ -*- 
// qsort.cc --- sort function for vector of the STL
//             algorithms from http://www.geocities.com/SoHo/2167/book.html
// Author          : Nicolas Anquetil
// Created On      : Mon Aug 25 10:36:40 1997
// 

# define T_ELT SoftwareObject*
# define GT(a,b) (compar((a),(b)) > 0)


void InsertSort(Vector<T_ELT> *array, int lb,int ub,
				 int (*compar)(T_ELT, T_ELT))
// --------------------------------------------------
{
  T_ELT insert;
  int i, j, jmin;

   /************************
    *  sort array[lb..ub]  *
    ************************/
    jmin = lb - 1;
    for (i = lb+1; i <= ub; i++) {
        insert = array->at(i);

        // shift elements down until insertion point found
        for (j = i-1; (j > jmin) && GT(array->at(j), insert); j--)
            array->at(j+1, array->at(j));
        array->at(j+1, insert);
    }
}


int partition(Vector<T_ELT> *array, int lb,int ub,
				 int (*compar)(T_ELT, T_ELT))
// --------------------------------------------------
{
  T_ELT exch;
  T_ELT pivot;
  int i, j, ipivot;
  unsigned int offset;

   /*****************************
    *  partition array[lb..ub]  *
    *****************************/

  // select pivot and exchange with 1st element
    offset = (ub - lb)>>1;
	ipivot = lb+offset;
    pivot = array->at(ipivot);
	array->at(ipivot, array->at(lb));

	i = lb+1;
	j = ub;
    while (true) {
        while ( (i < j)  && GT(pivot, array->at(i)) )
		  i++;
        while ( (j >= i) && GT(array->at(j), pivot) )
		  j--;

		if (i >= j)
		  break;

        exch = array->at(i);
        array->at(i, array->at(j));
        array->at(j, exch);

		i++;  j--;
    }

    // pivot belongs in array[j]
    array->at(lb, array->at(j));
    array->at(j, pivot);

    return(j);
}


void QuickSort(Vector<T_ELT> *array, int lb,int ub,
				 int (*compar)(T_ELT, T_ELT))
// --------------------------------------------------
{
  int m;

  /**************************
   *  Sort array[lb..ub]  *
   **************************/

  while (lb < ub) {

	// quickly sort short lists
	if (ub - lb <= 12) {
	  InsertSort(array, lb, ub, compar);
	  //BubbleSort(array, lb, ub, compar);
	  return;
	}

	// partition into two segments
	m = partition(array, lb, ub, compar);

	// sort the smallest partition
	// to minimize stack requirements
	if ( (m - lb) <= (ub - m) ) {
	  QuickSort(array, lb, m-1, compar);
	  lb = m + 1;
	}
	else {
	  QuickSort(array, m + 1, ub, compar);
	  ub = m - 1;
	}
  }
}
