/*                              -*- Mode: C++ -*- 
 * read-API.hh --- API to "Read" the new DB
 * Author          : Nicolas Anquetil
 * Created On      : Fri Jan 24 17:43:40 1997
 */ 

/* This file should be readable in C, beware to the comments */


# ifndef READAPI_H
# define READAPI_H

# ifdef __cplusplus
extern "C" {
# endif
# include "c++_c.h"
# include "utilFileMem.h"
# ifdef __cplusplus
}
# endif

# ifdef __cplusplus
# include "utils.hh"
# include "SourceUnit.hh"
# include "Definition.hh"
# include "ReferenceExistence.hh"
# define Iterator VoidIterator<SoftwareObject*>
# else

# define Iterator void
# endif   /* #ifdef __cplusplus */


/* CONNECT/DISCONNECT functions */
  /* quasi-synonym */
# define cdbInitReadOnlyFileMem(name) initGrowableFileMem(name,UFM_READ, 0, 0)
# ifndef cdbTerminateFileMem
# define cdbTerminateFileMem terminateFileMem
# endif

/* STATISTICS */
EXTERNC int   cdbGetObjectCount(class_t theClass);
EXTERNC char *cdbVersion();

/* INTERNAL QUERIES */
EXTERNC SoftwareObject *cdbGetObjectGivenKey(int key);
EXTERNC Iterator *cdbGetObjectListGivenKey(int key);
EXTERNC int cdbGetKeyGivenObject(SoftwareObject *object);
EXTERNC class_t cdbGetClassGivenObject(SoftwareObject *object);
EXTERNC Iterator *cdbGetDefList(SoftwareObject *object);

/* GLOBAL QUERIES */
EXTERNC Iterator *cdbGetObjectListGivenName(char *name, class_t theClass);
EXTERNC Iterator *cdbGetObjectListRegExp(char *regexp, class_t theClass);
EXTERNC Iterator *cdbGetObjectList(class_t theClass);

/* ASSOCIATION QUERIES */
EXTERNC Iterator *cdbGetObjectListThatReferToObject(class_t theClass,
						    SoftwareObject *object);
EXTERNC Iterator *cdbGetObjectListReferedByObject(class_t theClass,
						  SoftwareObject *object);
EXTERNC Iterator *cdbGetObjectListThatDefineObject(class_t theClass,
						  SoftwareObject *object);
EXTERNC Iterator *cdbGetObjectListDefinedByObject(class_t theClass,
						  SoftwareObject *object);

/* ATTRIBUTE QUERIES */
EXTERNC char *cdbGetObjectName(SoftwareObject *object);
EXTERNC void cdbGetSoftwareObjectAttributes(SoftwareObject *object,int *objectKey);
EXTERNC void cdbGetSourceUnitAttributes(SourceUnit *object,
					Iterator **contains,
					Iterator **refersToData,
					Iterator **refersToCommentTerm,
					Iterator **defines);
EXTERNC void cdbGetSourceFileAttributes(SourceFile *object,
					char *fileName,
					char *path,
					int *vers,
					int *dateChanged,
					int *size,
					Iterator **includes);
EXTERNC void cdbGetSourceWithinFileAttributes(SourceWithinFile *object,
					      SourceUnit **containingSource,
					      int *startChar,
					      int *endChar);
EXTERNC void cdbGetRoutineSourceAttributes(RoutineSource *object,
					   char *Routinename,
					   bool *Isentry,
					   TypeUseExistence **returnType,
					   Iterator **calls,
					   char *defby);
EXTERNC void cdbGetDefinitionAttributes(Definition *object,
					 char *definitionName);
EXTERNC void cdbGetTypedDefinitionAttributes(TypedDefinition *object,				                            TypeUseExistence **type);
EXTERNC void cdbGetStandaloneDefinitionAttributes(StandaloneDefinition *object,
						  int *startChar,
						  int *endChar,
						  SourceUnit **containingSource);		
EXTERNC void cdbGetFieldAttributes(Field *object,
				   RecordTypeDef **isFieldMemberOf);
EXTERNC void cdbGetDatumDefAttributes(DatumDef *object,
				      bool *isConst,
				      bool *isFormalArgument);
EXTERNC void cdbGetEnumerationConstAttributes(EnumerationConst *object,
					       EnumeratedTypeDef**isEnumerationMemberOf);
EXTERNC void cdbGetEnumeratedTypeDefAttributes(EnumeratedTypeDef *object,
					       Iterator **enumerationMembers);
EXTERNC void cdbGetRecordTypeDefAttributes(RecordTypeDef *object,
					   Iterator **fieldMembers);
EXTERNC void cdbGetReferenceExistenceAttributes(ReferenceExistence *object,
						char *referenceName);
EXTERNC void cdbGetCommentTermExistenceAttributes(CommentTermExistence *object,
						  SourceUnit **foundInSource,
						  Iterator **successorInCompound,
						  Iterator **predecessorInCompound);
EXTERNC void cdbGetFileInclusionExistenceAttributes(FileInclusionExistence *object,
						    Iterator **potentiallyIncudedIn);
EXTERNC void cdbDataUseExistenceAttributes(DataUseExistence *object,
					   Iterator **foundInSource,
					   Iterator **successorInCompound,
					   Iterator **predecessorInCompound);
EXTERNC void cdbTypeUseExistenceAttributes(TypeUseExistence *object,
					   Iterator **usedAsTypeOf,
					   Iterator **usedAsReturnTypeOf);
EXTERNC void cdbRoutineCallExistenceAttributes(RoutineCallExistence *object,
					      Iterator **potentiallyCalledBy); 
					   		  
					   				

/*EXTERNC void cdbGet<a class name>Attributes(SoftwareObject *object, char **name,
									<all other class attributes>);
									*/


/**************************************************
                 I T E R A T O R S
 **************************************************/

EXTERNC SoftwareObject *iteratorNext(Iterator *iter);
EXTERNC bool iteratorEOI(Iterator *iter);
EXTERNC void iteratorDestroy(Iterator *iter);
EXTERNC int iteratorHowManyLeft(Iterator *iter);

# endif
