//                              -*- Mode: C++ -*-
// utils.cc --- utilities for using the new DB
// Author          : Nicolas Anquetil
// Created On      : Fri Dec 13 17:49:18 1996
//

# include "utils.hh"
# include "SourceUnit.hh"
# include "ReferenceExistence.hh"
# include "Definition.hh"
# include "utilFileMem.h"



extern "C" void init_pointers()
//--------------------------------------------------
{
  the_DB = new Database(DBVERSION);
# ifdef DEBUG
  printf("init_pointers: the_DB=0x%x\n",the_DB);
# endif
}  /* init_pointers() */


extern "C" void init_pointers_from_db()
//--------------------------------------------------
{
  the_DB = (Database *)getFirstBlockOfFileMem();
# ifdef DEBUG
  printf("init_pointers_from_db: the_DB=0x%x\n",the_DB);
  the_DB->hello();
# endif
}  /* init_pointers_from_db() */


extern "C" char *hashRecKeyFunc(char *hashedObject)
//--------------------------------------------------
// Tim: dummy
{
printf("DUMMY KEY FUNC %x \n", hashedObject);
  return NULL;
}  /* hashRecKeyFunc() */


VoidIterator<SoftwareObject*> *getNamedObject(char *name,
//--------------------------------------------------
											    class_t filterClass)
{
  Name *nameObj;
  VoidIterator<SoftwareObject*> *itFound;

  if ( name == NULL )
    // no name
	itFound = new VoidIterator<SoftwareObject*>();
  else {
	nameObj = the_DB->findExistingName(name);

	if ( nameObj != NULL )
	  itFound = nameObj->getObjectList(filterClass);
	else
	  itFound = new VoidIterator<SoftwareObject*>();
  }

  return( itFound);
}  /* getNamedObject() */


int
SOcmp_p(SoftwareObject **x, SoftwareObject **y)
//--------------------------------------------------
// an interface between SO_strict_cmp() and qsort()
{
  return(SO_strict_cmp(*x,*y));
}


int
SO_strict_cmp(SoftwareObject *x, SoftwareObject *y)
//--------------------------------------------------
// tests strict equality  rather than just comptability
// use any order, e.g. rank in enumerated type same as Qeltcmp in the query language module
{
  char *xname, *yname;
  int   namecmp;

  xname = x->objectName();
  yname = y->objectName();

  if ( (xname != NULL) && (yname != NULL) ) {
    namecmp = strcmp(xname, yname);
	 if      (namecmp != 0)
	   return( namecmp );   // if names differ return result of strcmp
	 // else if names are same return the result of comparing classes 
	 else if (x->classType() == y->classType() )
	   return( 0 );    //  x & y are "equal"
	     // if not equal, set return value according to a comparison
	     // of their classes    
	 else if ((int)x->classType() < (int)y->classType()) 
	   return (-1);	  
	 else
	   return( 1);
  }

  else { fprintf(stderr,
		 "** Comparing objects of virtual class (with no name)\n");
  exit(2);
  }
}  /* SO_strict_cmp */


int
SO_loose_cmp(SoftwareObject *x, SoftwareObject *y)
//--------------------------------------------------
// tests comptability rather than equality
// same as Qeltcmp in the query language module
{
  char *xname, *yname;
  int   namecmp;

  xname = x->objectName();
  yname = y->objectName();

  if ( (xname != NULL) && (yname != NULL) ) {
	 namecmp = strcmp(xname, yname);
	if      (namecmp != 0)
	  return( namecmp);
	else if (compatibleclass(x->classType(), y->classType()))
	  return( 0);  /* x & y are "equal", i.e. compatible */

	/* use any order, e.g. rank in enumerated type */
	else if ((int)virtualizeclass(x->classType()) <
			 (int)virtualizeclass(y->classType()))
	  return( -1);
	else
	  return( 1); 
  }
  else {
	fprintf(stderr,"** Comparing objects of virtual class (with no name)\n");
	exit(2);
  }
}  /* SO_loose_cmp */


SoftwareObject *
SOmerge(SoftwareObject *x, SoftwareObject *y)
//--------------------------------------------------
// Merging two equal SoftwareObject
{
  return(x);
}


void
sortTheList( List<SoftwareObject*> *l)
// --------------------------------------------------
{
  SoftwareObject **array;
  LIterator<SoftwareObject*> *iter;
  int i, nbElt;

  iter = new LIterator<SoftwareObject*>(l);
  nbElt = iter->howManyLeft();

  // transfer the list to an array
  array = (SoftwareObject**)malloc( nbElt*sizeof(SoftwareObject*));
  for ( i=0; ! iter->eoi(); i++)
	array[i] = iter->next();

  l->clear();        // empty the list (not the SO it contains
  delete(iter);

  // sort the array
  qsort( array, nbElt, sizeof(SoftwareObject*),
		 (int(*)(const void*,const void*))SOcmp_p);

  // recreate the list from the array (sorted)
  for (i=0; i < nbElt; i++)
	l->add(array[i]);

  free(array);
}
