//                              -*- Mode: C++ -*-
// write-API.cc --- API to create the new DB
// Author          : Nicolas Anquetil
// Created On      : Thu Dec 12 17:22:14 1996
//

# include "write-API.hh"

SourceFile*
cdbCreateSourceFile(
// --------------------------------------------------
    char *name,
    int   version,
    char *path,
    int   dateChanged,
    int   size)
{
  char *p;

  // strips trailing '/' in the path
  for ( p=path; *p; p++ ) ;
  p--;
  if (*p == '/')
    *p = '\0';

  return( new SourceFile(name,version,path,dateChanged,size));
}  /* cdbCreateSourceFile */


RoutineSource*
cdbCreateRoutineSource(
// --------------------------------------------------
  SourceUnit *containingSource,
  char *name,
  char *optReturnTypeName,
  int startChar,
  int endChar,
  bool isEntry)
{
  TypeUseExistence *typeUsed;
  RoutineSource    *newRoutine;

  if (optReturnTypeName == NULL)
    typeUsed = NULL;
  else
	typeUsed = (TypeUseExistence *)cdbRegisterDataUse( containingSource,
													   optReturnTypeName,
													   NULL, false, true);

  newRoutine = new RoutineSource( containingSource, startChar, endChar,
                                  name, isEntry, typeUsed );

  return( newRoutine);
}  /* cdbCreateRoutineSource */


DatumDef*
cdbCreateDatumDef(
// --------------------------------------------------
  SourceUnit *fileOrRoutine,
  char *name,
  bool isFormalArgument,
  bool isConstant,
  char *optTypeName,
  int startChar,
  int endChar)
{
  TypeUseExistence *typeUsed;
  DatumDef         *newDatum;

  if (optTypeName == NULL)
    typeUsed = NULL;
  else
	typeUsed = (TypeUseExistence *)cdbRegisterDataUse( fileOrRoutine,
													   optTypeName, NULL,
													   false, true);

  newDatum = new DatumDef( name, typeUsed, startChar, endChar,
                           fileOrRoutine, isConstant, isFormalArgument);

  return( newDatum);
}  /* cdbCreateDatumDef */


TypeDef*
cdbCreateTypeDef(
// --------------------------------------------------
  SourceUnit *containingSource,
  char *name,
  char *optTypeName,
  bool isRecord,
  bool isEnumerated,
  int startChar,
  int endChar)
{
  TypeUseExistence *typeUsed;
  TypeDef          *newType;

  if (optTypeName == NULL)
    typeUsed = NULL;
  else 
	typeUsed = (TypeUseExistence *)cdbRegisterDataUse( containingSource,
													   optTypeName, NULL,
													   false, true);

  if ( isRecord )
    newType = new RecordTypeDef( name, typeUsed, startChar, endChar,
                                 containingSource);
  else if ( isEnumerated )
    newType = new EnumeratedTypeDef( name, typeUsed, startChar, endChar,
                                     containingSource);
  else
    newType = new TypeDef( name, typeUsed, startChar, endChar,
                           containingSource);

  return(newType);
}  /* cdbCreateTypeDef */


Field*
cdbCreateField(
// --------------------------------------------------
  RecordTypeDef *record,
  char *name,
  char *optTypeName)
{
  TypeUseExistence *typeUsed;

  if (optTypeName == NULL)
    typeUsed = NULL;
  else 
	typeUsed = (TypeUseExistence *)
	  cdbRegisterDataUse( record->containingSource(), optTypeName, NULL,
						  false, true);

  return( new Field( name, typeUsed, record) );
}  /* cdbCreateField */


EnumerationConst*
cdbCreateEnumerationConst(
// --------------------------------------------------
  EnumeratedTypeDef *enumeration,
  char *name)
{
  return( new EnumerationConst( name, enumeration) );
}  /* cdbCreateEnumerationConst */


RoutineCallExistence*
cdbRegisterRoutineCall(
// --------------------------------------------------
  SourceUnit *caller,
  char *calledRoutineName)
{
  VoidIterator<SoftwareObject*> *iter;
  RoutineCallExistence *retn;



	iter = getNamedObject( calledRoutineName, _routinecallexistence);

	if (iter->eoi())
	  retn = new RoutineCallExistence( calledRoutineName);
	else
	  retn = (RoutineCallExistence *)iter->next();

	delete( iter);

	retn->addCallingSource( caller);
	caller->addCalledRoutine( retn);

  return(retn);
}  /* cdbRegisterRoutineCall */


FileInclusionExistence*
cdbRegisterFileInclusion(
// --------------------------------------------------
  SourceFile *includerFile,
  char *includedFileName)
{
  FileInclusionExistence *retn;
  VoidIterator<SoftwareObject*> *iter;

  iter = getNamedObject( includedFileName, _fileinclusionexistence);

  if (iter->eoi())
	retn = new FileInclusionExistence( includedFileName);
  else
	retn = (FileInclusionExistence *)iter->next();

  delete( iter);

  retn->addInclusion( includerFile);
  includerFile->addIncludedFile( retn);

  return(retn);
}  /* cdbRegisterFileInclusion */


CommentTermExistence*
cdbRegisterCommentTerm(
// --------------------------------------------------
  SourceUnit *containingSource,
  char *name,
  CommentTermExistence *optPreviousTerm,
  bool isConditionalCompData)   // ??
{
  CommentTermExistence *retn;
  VoidIterator<SoftwareObject*> *iter;

  iter = getNamedObject(name, _commenttermexistence);

  if (iter->eoi()) {
	retn = new CommentTermExistence( name, containingSource,
									 isConditionalCompData);

	if (optPreviousTerm != NULL)
	  optPreviousTerm->addSuccessorInCompound( retn);
  }
  else
	retn = (CommentTermExistence *)iter->next();

  delete( iter);

  return(retn);
}  /* cdbRegisterCommentTerm */


DataUseExistence*
cdbRegisterDataUse(
// --------------------------------------------------
  SourceUnit *containingSource,
  char *dataName,
  DataUseExistence *optPreviousComponent,
  bool isManifestConstant,
  bool isTypeUse)
{
  DataUseExistence *retn;
  VoidIterator<SoftwareObject*> *iter;
  class_t      filter;

  if (isManifestConstant)
	filter = _manifestconstexistence;
  else if (isTypeUse)
	filter = _typeuseexistence;
  else
	filter = _datauseexistence;

  iter = getNamedObject(dataName, filter);

  if (iter->eoi()) {
	switch (filter) {
	case _manifestconstexistence:
	  retn = new ManifestConstExistence( dataName);
	  break;
	case _typeuseexistence:
	  retn = new TypeUseExistence( dataName);
	  break;
	case _datauseexistence:
	  retn = new DataUseExistence( dataName);
	  break;
	}

	if (optPreviousComponent != NULL)
	  optPreviousComponent->addSuccessorInCompound( retn);
  }
  else
	retn = (DataUseExistence *)iter->next();

  delete( iter);

  retn->addReferingSource(containingSource);
  containingSource->addReference(retn,false);  // false=is not a comment

  return(retn);
}  /* cdbRegisterDataUse */
